@@ -14,9 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GKD_GLUE_H_
@@ -27,6 +26,4 @@
 
 gboolean       gkd_daemon_startup_ssh      (void);
 
-gboolean       gkd_daemon_startup_gpg      (void);
-
 #endif /* GKD_GLUE_H_ */
