@@ -43,7 +43,11 @@ EGG_SECURE_DECLARE (data_der);
 
 static GQuark OID_PKIX1_RSA;
 static GQuark OID_PKIX1_DSA;
+static GQuark OID_PKIX1_ECDSA;
 static GQuark OID_PKCS12_PBE_3DES_SHA1;
+static GQuark OID_ANSI_SECP256R1;
+static GQuark OID_ANSI_SECP384R1;
+static GQuark OID_ANSI_SECP521R1;
 
 static void
 init_quarks (void)
@@ -57,7 +61,11 @@ init_quarks (void)
 
 		QUARK (OID_PKIX1_RSA, "1.2.840.113549.1.1.1");
 		QUARK (OID_PKIX1_DSA, "1.2.840.10040.4.1");
+		QUARK (OID_PKIX1_ECDSA, "1.2.840.10045.2.1");
 		QUARK (OID_PKCS12_PBE_3DES_SHA1, "1.2.840.113549.1.12.1.3");
+		QUARK (OID_ANSI_SECP256R1, "1.2.840.10045.3.1.7");
+		QUARK (OID_ANSI_SECP384R1, "1.3.132.0.34");
+		QUARK (OID_ANSI_SECP521R1, "1.3.132.0.35");
 
 		#undef QUARK
 
@@ -65,6 +73,161 @@ init_quarks (void)
 	}
 }
 
+const gchar *
+gkm_data_der_oid_to_curve (GQuark oid)
+{
+	if (oid == OID_ANSI_SECP256R1)
+		return "NIST P-256";
+	else if (oid == OID_ANSI_SECP384R1)
+		return "NIST P-384";
+	else if (oid == OID_ANSI_SECP521R1)
+		return "NIST P-521";
+	return NULL;
+}
+
+/*
+ * Convert ecc->curve values from libgcrypt representation to internal one.
+ * Ignore duplicates and alternative names, since S-expressions are created
+ * only by us throught this code.
+ */
+static GQuark
+gkm_data_der_curve_to_oid (const gchar *curve)
+{
+	if (g_str_equal (curve, "NIST P-256"))
+		return OID_ANSI_SECP256R1;
+	else if (g_str_equal (curve, "NIST P-384"))
+		return OID_ANSI_SECP384R1;
+	else if (g_str_equal (curve, "NIST P-521"))
+		return OID_ANSI_SECP521R1;
+	return 0;
+}
+
+
+GQuark
+gkm_data_der_oid_from_ec_params (GBytes *params)
+{
+	GNode *asn;
+	GQuark oid;
+
+	init_quarks ();
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "Parameters", params);
+	if (!asn)
+		return 0;
+
+	oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, "namedCurve", NULL));
+
+	egg_asn1x_destroy (asn);
+	return oid;
+}
+
+GBytes *
+gkm_data_der_get_ec_params (GQuark oid)
+{
+	GNode *asn;
+	GBytes *params = NULL;
+	GNode *named_curve;
+
+	asn = egg_asn1x_create (pk_asn1_tab, "Parameters");
+	if (!asn)
+		goto done;
+
+	named_curve = egg_asn1x_node (asn, "namedCurve", NULL);
+
+	if (!egg_asn1x_set_oid_as_quark (named_curve, oid))
+		goto done;
+
+	if (!egg_asn1x_set_choice (asn, named_curve))
+		goto done;
+
+	params = egg_asn1x_encode (asn, NULL);
+
+done:
+	egg_asn1x_destroy (asn);
+	return params;
+}
+
+/* wrapper so we do not have to export the GQuark magic */
+GBytes *
+gkm_data_der_curve_to_ec_params (const gchar *curve_name)
+{
+	GQuark oid;
+
+	init_quarks ();
+
+	oid = gkm_data_der_curve_to_oid (curve_name);
+	if (oid == 0)
+		return NULL;
+
+	return gkm_data_der_get_ec_params (oid);
+}
+
+GBytes *
+gkm_data_der_encode_ecdsa_q_str (const guchar *data, gsize data_len)
+{
+	GNode *asn = NULL;
+	GBytes *bytes, *result = NULL;
+
+	asn = egg_asn1x_create (pk_asn1_tab, "ECKeyQ");
+	g_return_val_if_fail (asn, FALSE);
+
+	bytes = g_bytes_new_static (data, data_len);
+
+	/* "consumes" bytes */
+	if (!gkm_data_asn1_write_string (asn, bytes))
+		goto done;
+
+	result = egg_asn1x_encode (asn, g_realloc);
+	if (result == NULL)
+		g_warning ("couldn't encode Q into the PKCS#11 structure: %s", egg_asn1x_message (asn));
+done:
+	egg_asn1x_destroy (asn);
+	return result;
+}
+
+gboolean
+gkm_data_der_encode_ecdsa_q (gcry_mpi_t q, GBytes **result)
+{
+	gcry_error_t gcry;
+	guchar data[1024];
+	gsize data_len = 1024;
+	gboolean rv = TRUE;
+
+	g_assert (q);
+	g_assert (result);
+
+	gcry = gcry_mpi_print (GCRYMPI_FMT_USG, data, data_len, &data_len, q);
+	g_return_val_if_fail (gcry == 0, FALSE);
+
+	*result = gkm_data_der_encode_ecdsa_q_str (data, data_len);
+	if (*result == NULL)
+		rv = FALSE;
+
+	return rv;
+}
+
+gboolean
+gkm_data_der_decode_ecdsa_q (GBytes *data, GBytes **result)
+{
+	GNode *asn = NULL;
+	gboolean rv = TRUE;
+
+	g_assert (data);
+	g_assert (result);
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "ECKeyQ", data);
+	/* workaround a bug in gcr (not DER encoding the MPI) */
+	if (!asn) {
+		*result = data;
+		return rv;
+	}
+
+	rv = gkm_data_asn1_read_string (asn, result);
+
+	egg_asn1x_destroy (asn);
+	return rv;
+}
+
 /* -----------------------------------------------------------------------------
  * KEY PARSING
  */
@@ -402,12 +565,124 @@ done:
 	return ret;
 }
 
+#define SEXP_PUBLIC_ECDSA  \
+	"(public-key"   \
+	"  (ecdsa"         \
+	"    (curve %s)"     \
+	"    (q %b)))"
+
+GkmDataResult
+gkm_data_der_read_public_key_ecdsa (GBytes *data,
+                                     gcry_sexp_t *s_key)
+{
+	GkmDataResult ret = GKM_DATA_UNRECOGNIZED;
+	int res;
+	GNode *asn = NULL;
+	GBytes *q = NULL;
+	gsize q_bits;
+	GQuark oid;
+	const gchar *curve = NULL;
+
+	init_quarks ();
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "ECPublicKey", data);
+	if (!asn)
+		goto done;
+
+	ret = GKM_DATA_FAILURE;
+
+	if (!gkm_data_asn1_read_oid (egg_asn1x_node (asn, "parameters", "namedCurve", NULL), &oid) ||
+	    !gkm_data_asn1_read_bit_string (egg_asn1x_node (asn, "q", NULL), &q, &q_bits))
+		goto done;
+
+	curve = gkm_data_der_oid_to_curve (oid);
+	if (curve == NULL)
+		goto done;
+
+	res = gcry_sexp_build (s_key, NULL, SEXP_PUBLIC_ECDSA,
+                               curve,
+                               g_bytes_get_size(q), g_bytes_get_data(q, NULL));
+	if (res)
+		goto done;
+
+	g_assert (*s_key);
+	ret = GKM_DATA_SUCCESS;
+
+done:
+	egg_asn1x_destroy (asn);
+	g_bytes_unref (q);
+
+	if (ret == GKM_DATA_FAILURE)
+		g_message ("invalid ECDSA key");
+
+	return ret;
+}
+
+#define SEXP_PRIVATE_ECDSA  \
+	"(private-key"   \
+	"  (ecdsa"         \
+	"    (curve %s)"     \
+	"    (q %b)"     \
+	"    (d %m)))"
+
+GkmDataResult
+gkm_data_der_read_private_key_ecdsa (GBytes *data,
+                                     gcry_sexp_t *s_key)
+{
+	gcry_mpi_t d = NULL;
+	GkmDataResult ret = GKM_DATA_UNRECOGNIZED;
+	int res;
+	GNode *asn = NULL;
+	GBytes *q = NULL;
+	gsize q_bits;
+	GQuark oid;
+	const gchar *curve = NULL;
+
+	init_quarks ();
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "ECPrivateKey", data);
+	if (!asn)
+		goto done;
+
+	ret = GKM_DATA_FAILURE;
+
+	if (!gkm_data_asn1_read_string_mpi (egg_asn1x_node (asn, "d", NULL), &d) ||
+	    !gkm_data_asn1_read_oid (egg_asn1x_node (asn, "parameters", "namedCurve", NULL), &oid) ||
+	    !gkm_data_asn1_read_bit_string (egg_asn1x_node (asn, "q", NULL), &q, &q_bits))
+		goto done;
+
+	curve = gkm_data_der_oid_to_curve (oid);
+	if (curve == NULL)
+		goto done;
+
+	res = gcry_sexp_build (s_key, NULL, SEXP_PRIVATE_ECDSA,
+                               curve,
+                               g_bytes_get_size(q), g_bytes_get_data(q, NULL), d);
+	if (res)
+		goto done;
+
+	g_assert (*s_key);
+	ret = GKM_DATA_SUCCESS;
+
+done:
+	egg_asn1x_destroy (asn);
+	gcry_mpi_release (d);
+	g_bytes_unref (q);
+
+	if (ret == GKM_DATA_FAILURE)
+		g_message ("invalid ECDSA key");
+
+	return ret;
+}
+
 GkmDataResult
 gkm_data_der_read_public_key (GBytes *data, gcry_sexp_t *s_key)
 {
 	GkmDataResult res;
 
 	res = gkm_data_der_read_public_key_rsa (data, s_key);
+	if (res == GKM_DATA_UNRECOGNIZED)
+		res = gkm_data_der_read_public_key_ecdsa (data, s_key);
 	if (res == GKM_DATA_UNRECOGNIZED)
 		res = gkm_data_der_read_public_key_dsa (data, s_key);
 
@@ -459,6 +734,10 @@ gkm_data_der_read_public_key_info (GBytes *data,
 		ret = gkm_data_der_read_public_key_dsa_parts (key, params, s_key);
 		g_bytes_unref (params);
 
+	/* A ECDSA key */
+	} else if (oid == OID_PKIX1_ECDSA) {
+		ret = gkm_data_der_read_public_key_ecdsa (key, s_key);
+
 	} else {
 		g_message ("unsupported key algorithm in certificate: %s", g_quark_to_string (oid));
 		ret = GKM_DATA_UNRECOGNIZED;
@@ -485,6 +764,8 @@ gkm_data_der_read_private_key (GBytes *data,
 	res = gkm_data_der_read_private_key_rsa (data, s_key);
 	if (res == GKM_DATA_UNRECOGNIZED)
 		res = gkm_data_der_read_private_key_dsa (data, s_key);
+	if (res == GKM_DATA_UNRECOGNIZED)
+		res = gkm_data_der_read_private_key_ecdsa (data, s_key);
 
 	return res;
 }
@@ -518,6 +799,8 @@ gkm_data_der_read_private_pkcs8_plain (GBytes *data,
 		algorithm = GCRY_PK_RSA;
 	else if (key_algo == OID_PKIX1_DSA)
 		algorithm = GCRY_PK_DSA;
+	else if (key_algo == OID_PKIX1_ECDSA)
+		algorithm = GCRY_PK_ECC;
 
 	if (!algorithm) {
 		ret = GKM_DATA_UNRECOGNIZED;
@@ -546,6 +829,9 @@ done:
 			if (ret == GKM_DATA_UNRECOGNIZED && params)
 				ret = gkm_data_der_read_private_key_dsa_parts (keydata, params, s_key);
 			break;
+		case GCRY_PK_ECC:
+			ret = gkm_data_der_read_private_key_ecdsa (keydata, s_key);
+			break;
 		default:
 			g_message ("invalid or unsupported key type in PKCS#8 key");
 			ret = GKM_DATA_UNRECOGNIZED;
@@ -914,6 +1200,113 @@ done:
 	return result;
 }
 
+GBytes *
+gkm_data_der_write_public_key_ecdsa (gcry_sexp_t s_key)
+{
+	GNode *asn = NULL, *named_curve;
+	gcry_mpi_t d = NULL;
+	GBytes *result = NULL, *q = NULL;
+	gchar *q_data = NULL;
+	GQuark oid;
+	gchar *curve = NULL;
+	gsize q_size;
+
+	init_quarks ();
+
+	asn = egg_asn1x_create (pk_asn1_tab, "ECPublicKey");
+	g_return_val_if_fail (asn, NULL);
+
+	if (!gkm_sexp_extract_buffer (s_key, &q_data, &q_size, "ecdsa", "q", NULL) ||
+	    !gkm_sexp_extract_string (s_key, &curve, "ecdsa", "curve", NULL))
+		goto done;
+
+	oid = gkm_data_der_curve_to_oid (curve);
+	g_free (curve);
+	if (oid == 0)
+		goto done;
+
+	q = g_bytes_new_take (q_data, q_size);
+	if (q == NULL)
+		goto done;
+
+	named_curve = egg_asn1x_node (asn, "parameters", "namedCurve", NULL);
+
+	/* XXX the bit size does not have to match byte size * 8 exactly
+	 * it would be good to cover this with more tests
+	 */
+	if (!gkm_data_asn1_write_bit_string (egg_asn1x_node (asn, "q", NULL), q, q_size*8) ||
+	    !gkm_data_asn1_write_oid (named_curve, oid))
+		goto done;
+
+	if (!egg_asn1x_set_choice (egg_asn1x_node (asn, "parameters", NULL), named_curve))
+		goto done;
+
+	result = egg_asn1x_encode (asn, egg_secure_realloc);
+	if (result == NULL)
+		g_warning ("couldn't encode public ecdsa key: %s", egg_asn1x_message (asn));
+
+done:
+	egg_asn1x_destroy (asn);
+	gcry_mpi_release (d);
+	g_bytes_unref (q);
+
+	return result;
+}
+
+GBytes *
+gkm_data_der_write_private_key_ecdsa (gcry_sexp_t s_key)
+{
+	GNode *asn = NULL, *named_curve;
+	gcry_mpi_t d = NULL;
+	GBytes *result = NULL, *q = NULL;
+	gchar *q_data = NULL;
+	GQuark oid;
+	gchar *curve = NULL;
+	gsize q_size;
+
+	init_quarks ();
+
+	asn = egg_asn1x_create (pk_asn1_tab, "ECPrivateKey");
+	g_return_val_if_fail (asn, NULL);
+
+	if (!gkm_sexp_extract_mpi (s_key, &d, "ecdsa", "d", NULL) ||
+	    !gkm_sexp_extract_buffer (s_key, &q_data, &q_size, "ecdsa", "q", NULL) ||
+	    !gkm_sexp_extract_string (s_key, &curve, "ecdsa", "curve", NULL))
+		goto done;
+
+	oid = gkm_data_der_curve_to_oid (curve);
+	g_free (curve);
+	if (oid == 0)
+		goto done;
+
+	q = g_bytes_new_take (q_data, q_size);
+	if (q == NULL)
+		goto done;
+
+	named_curve = egg_asn1x_node (asn, "parameters", "namedCurve", NULL);
+
+	if (!gkm_data_asn1_write_string_mpi (egg_asn1x_node (asn, "d", NULL), d) ||
+	    !gkm_data_asn1_write_bit_string (egg_asn1x_node (asn, "q", NULL), q, q_size*8) ||
+	    !gkm_data_asn1_write_oid (named_curve, oid))
+		goto done;
+
+	if (!egg_asn1x_set_choice (egg_asn1x_node (asn, "parameters", NULL), named_curve))
+		goto done;
+
+	egg_asn1x_set_integer_as_ulong (egg_asn1x_node (asn, "version", NULL), 1);
+
+	result = egg_asn1x_encode (asn, egg_secure_realloc);
+	if (result == NULL)
+		g_warning ("couldn't encode private ecdsa key: %s", egg_asn1x_message (asn));
+
+done:
+	egg_asn1x_destroy (asn);
+	gcry_mpi_release (d);
+	g_bytes_unref (q);
+
+	return result;
+}
+
 GBytes *
 gkm_data_der_write_public_key (gcry_sexp_t s_key)
 {
@@ -932,6 +1325,8 @@ gkm_data_der_write_public_key (gcry_sexp_t s_key)
 		return gkm_data_der_write_public_key_rsa (s_key);
 	case GCRY_PK_DSA:
 		return gkm_data_der_write_public_key_dsa (s_key);
+	case GCRY_PK_ECC:
+		return gkm_data_der_write_public_key_ecdsa (s_key);
 	default:
 		g_return_val_if_reached (NULL);
 	}
@@ -955,6 +1350,8 @@ gkm_data_der_write_private_key (gcry_sexp_t s_key)
 		return gkm_data_der_write_private_key_rsa (s_key);
 	case GCRY_PK_DSA:
 		return gkm_data_der_write_private_key_dsa (s_key);
+	case GCRY_PK_ECC:
+		return gkm_data_der_write_private_key_ecdsa (s_key);
 	default:
 		g_return_val_if_reached (NULL);
 	}
@@ -1063,6 +1460,12 @@ gkm_data_der_write_private_pkcs8_plain (gcry_sexp_t skey)
 		params = gkm_data_der_write_private_key_dsa_params (skey);
 		break;
 
+	case GCRY_PK_ECC:
+		oid = OID_PKIX1_ECDSA;
+		params = NULL;
+		key = gkm_data_der_write_private_key_ecdsa (skey);
+		break;
+
 	default:
 		g_warning ("trying to serialize unsupported private key algorithm: %d", algorithm);
 		return NULL;
